@@ -1,42 +1,50 @@
 """Sub-module providing sequence-formatting functions."""
-# standard imports
-import curses
+# std imports
+import platform
 
-# 3rd-party
+# 3rd party
 import six
 
+# local
+from blessed.colorspace import CGA_COLORS, X11_COLORNAMES_TO_RGB
+
+# curses
+if platform.system() == 'Windows':
+    import jinxed as curses   # pylint: disable=import-error
+else:
+    import curses
+
 
 def _make_colors():
     """
     Return set of valid colors and their derivatives.
 
     :rtype: set
+    :returns: Color names with prefixes
     """
-    derivatives = ('on', 'bright', 'on_bright',)
-    colors = set('black red green yellow blue magenta cyan white'.split())
-    return set('_'.join((_deravitive, _color))
-               for _deravitive in derivatives
-               for _color in colors) | colors
+    colors = set()
+    # basic CGA foreground color, background, high intensity, and bold
+    # background ('iCE colors' in my day).
+    for cga_color in CGA_COLORS:
+        colors.add(cga_color)
+        colors.add('on_' + cga_color)
+        colors.add('bright_' + cga_color)
+        colors.add('on_bright_' + cga_color)
+
+    # foreground and background VGA color
+    for vga_color in X11_COLORNAMES_TO_RGB:
+        colors.add(vga_color)
+        colors.add('on_' + vga_color)
+    return colors
 
 
-def _make_compoundables(colors):
-    """
-    Return given set ``colors`` along with all "compoundable" attributes.
-
-    :arg set colors: set of color names as string.
-    :rtype: set
-    """
-    _compoundables = set('bold underline reverse blink dim italic shadow '
-                         'standout subscript superscript'.split())
-    return colors | _compoundables
-
-
-#: Valid colors and their background (on), bright,
-#: and bright-background derivatives.
+#: Valid colors and their background (on), bright, and bright-background
+#: derivatives.
 COLORS = _make_colors()
 
-#: Attributes and colors which may be compounded by underscore.
-COMPOUNDABLES = _make_compoundables(COLORS)
+#: Attributes that may be compounded with colors, by underscore, such as
+#: 'reverse_indigo'.
+COMPOUNDABLES = set('bold underline reverse blink italic standout'.split())
 
 
 class ParameterizingString(six.text_type):
@@ -45,24 +53,25 @@ class ParameterizingString(six.text_type):
 
     For example::
 
+        >>> from blessed import Terminal
         >>> term = Terminal()
         >>> color = ParameterizingString(term.color, term.normal, 'color')
         >>> color(9)('color #9')
         u'\x1b[91mcolor #9\x1b(B\x1b[m'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, cap, normal=u'', name=u'<not specified>'):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 3 positional arguments.
 
-        :arg cap: parameterized string suitable for curses.tparm()
-        :arg normal: terminating sequence for this capability (optional).
-        :arg name: name of this terminal capability (optional).
+        :arg str cap: parameterized string suitable for curses.tparm()
+        :arg str normal: terminating sequence for this capability (optional).
+        :arg str name: name of this terminal capability (optional).
         """
-        assert args and len(args) < 4, args
-        new = six.text_type.__new__(cls, args[0])
-        new._normal = args[1] if len(args) > 1 else u''
-        new._name = args[2] if len(args) > 2 else u'<not specified>'
+        new = six.text_type.__new__(cls, cap)
+        new._normal = normal
+        new._name = name
         return new
 
     def __call__(self, *args):
@@ -73,7 +82,10 @@ class ParameterizingString(six.text_type):
         ``*args``, followed by the terminating sequence (self.normal) into
         a :class:`FormattingString` capable of being called.
 
+        :raises TypeError: Mismatch between capability and arguments
+        :raises curses.error: :func:`curses.tparm` raised an exception
         :rtype: :class:`FormattingString` or :class:`NullCallableString`
+        :returns: Callable string for given parameters
         """
         try:
             # Re-encode the cap, because tparm() takes a bytestring in Python
@@ -86,10 +98,8 @@ class ParameterizingString(six.text_type):
             # something intelligent:
             if args and isinstance(args[0], six.string_types):
                 raise TypeError(
-                    "A native or nonexistent capability template, %r received"
-                    " invalid argument %r: %s. You probably misspelled a"
-                    " formatting call like `bright_red'" % (
-                        self._name, args, err))
+                    "Unknown terminal capability, %r, or, TypeError "
+                    "for arguments %r: %s" % (self._name, args, err))
             # Somebody passed a non-string; I don't feel confident
             # guessing what they were trying to do.
             raise
@@ -126,22 +136,23 @@ class ParameterizingProxyString(six.text_type):
         u'\x1b[10G'
     """
 
-    def __new__(cls, *args):
+    def __new__(cls, fmt_pair, normal=u'', name=u'<not specified>'):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
         """
         Class constructor accepting 4 positional arguments.
 
-        :arg fmt: format string suitable for displaying terminal sequences.
-        :arg callable: receives __call__ arguments for formatting fmt.
-        :arg normal: terminating sequence for this capability (optional).
-        :arg name: name of this terminal capability (optional).
+        :arg tuple fmt_pair: Two element tuple containing:
+            - format string suitable for displaying terminal sequences
+            - callable suitable for receiving  __call__ arguments for formatting string
+        :arg str normal: terminating sequence for this capability (optional).
+        :arg str name: name of this terminal capability (optional).
         """
-        assert args and len(args) < 4, args
-        assert isinstance(args[0], tuple), args[0]
-        assert callable(args[0][1]), args[0][1]
-        new = six.text_type.__new__(cls, args[0][0])
-        new._fmt_args = args[0][1]
-        new._normal = args[1] if len(args) > 1 else u''
-        new._name = args[2] if len(args) > 2 else u'<not specified>'
+        assert isinstance(fmt_pair, tuple), fmt_pair
+        assert callable(fmt_pair[1]), fmt_pair[1]
+        new = six.text_type.__new__(cls, fmt_pair[0])
+        new._fmt_args = fmt_pair[1]
+        new._normal = normal
+        new._name = name
         return new
 
     def __call__(self, *args):
@@ -154,11 +165,155 @@ class ParameterizingProxyString(six.text_type):
         given capability.
 
         :rtype: FormattingString
+        :returns: Callable string for given parameters
         """
         return FormattingString(self.format(*self._fmt_args(*args)),
                                 self._normal)
 
 
+class FormattingString(six.text_type):
+    r"""
+    A Unicode string which doubles as a callable.
+
+    This is used for terminal attributes, so that it may be used both
+    directly, or as a callable.  When used directly, it simply emits
+    the given terminal sequence.  When used as a callable, it wraps the
+    given (string) argument with the 2nd argument used by the class
+    constructor::
+
+        >>> from blessed import Terminal
+        >>> term = Terminal()
+        >>> style = FormattingString(term.bright_blue, term.normal)
+        >>> print(repr(style))
+        u'\x1b[94m'
+        >>> style('Big Blue')
+        u'\x1b[94mBig Blue\x1b(B\x1b[m'
+    """
+
+    def __new__(cls, sequence, normal=u''):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
+        """
+        Class constructor accepting 2 positional arguments.
+
+        :arg str sequence: terminal attribute sequence.
+        :arg str normal: terminating sequence for this attribute (optional).
+        """
+        new = six.text_type.__new__(cls, sequence)
+        new._normal = normal
+        return new
+
+    def __call__(self, *args):
+        """
+        Return ``text`` joined by ``sequence`` and ``normal``.
+
+        :raises TypeError: Not a string type
+        :rtype: str
+        :returns: Arguments wrapped in sequence and normal
+        """
+        # Jim Allman brings us this convenience of allowing existing
+        # unicode strings to be joined as a call parameter to a formatting
+        # string result, allowing nestation:
+        #
+        # >>> t.red('This is ', t.bold('extremely'), ' dangerous!')
+        for idx, ucs_part in enumerate(args):
+            if not isinstance(ucs_part, six.string_types):
+                expected_types = ', '.join([
+                    _type.__name__ for _type in six.string_types])
+                raise TypeError(
+                    "TypeError for FormattingString argument, "
+                    "%r, at position %s: expected type %s, "
+                    "got %s" % (ucs_part, idx, expected_types,
+                                type(ucs_part).__name__))
+        postfix = u''
+        if self and self._normal:
+            postfix = self._normal
+            _refresh = self._normal + self
+            args = [_refresh.join(ucs_part.split(self._normal))
+                    for ucs_part in args]
+
+        return self + u''.join(args) + postfix
+
+
+class FormattingOtherString(six.text_type):
+    r"""
+    A Unicode string which doubles as a callable for another sequence when called.
+
+    This is used for the :meth:`~.Terminal.move_up`, ``down``, ``left``, and ``right()``
+    family of functions::
+
+        >>> from blessed import Terminal
+        >>> term = Terminal()
+        >>> move_right = FormattingOtherString(term.cuf1, term.cuf)
+        >>> print(repr(move_right))
+        u'\x1b[C'
+        >>> print(repr(move_right(666)))
+        u'\x1b[666C'
+        >>> print(repr(move_right()))
+        u'\x1b[C'
+    """
+
+    def __new__(cls, direct, target):
+        # pylint: disable = missing-return-doc, missing-return-type-doc
+        """
+        Class constructor accepting 2 positional arguments.
+
+        :arg str direct: capability name for direct formatting, eg ``('x' + term.right)``.
+        :arg str target: capability name for callable, eg ``('x' + term.right(99))``.
+        """
+        new = six.text_type.__new__(cls, direct)
+        new._callable = target
+        return new
+
+    def __getnewargs__(self):
+        # return arguments used for the __new__ method upon unpickling.
+        return six.text_type.__new__(six.text_type, self), self._callable
+
+    def __call__(self, *args):
+        """Return ``text`` by ``target``."""
+        if args:
+            return self._callable(*args)
+        return self
+
+
+class NullCallableString(six.text_type):
+    """
+    A dummy callable Unicode alternative to :class:`FormattingString`.
+
+    This is used for colors on terminals that do not support colors, it is just a basic form of
+    unicode that may also act as a callable.
+    """
+
+    def __new__(cls):
+        """Class constructor."""
+        new = six.text_type.__new__(cls, u'')
+        return new
+
+    def __call__(self, *args):
+        """
+        Allow empty string to be callable, returning given string, if any.
+
+        When called with an int as the first arg, return an empty Unicode. An
+        int is a good hint that I am a :class:`ParameterizingString`, as there
+        are only about half a dozen string-returning capabilities listed in
+        terminfo(5) which accept non-int arguments, they are seldom used.
+
+        When called with a non-int as the first arg (no no args at all), return
+        the first arg, acting in place of :class:`FormattingString` without
+        any attributes.
+        """
+        if not args or isinstance(args[0], int):
+            # As a NullCallableString, even when provided with a parameter,
+            # such as t.color(5), we must also still be callable, fe:
+            #
+            # >>> t.color(5)('shmoo')
+            #
+            # is actually simplified result of NullCallable()() on terminals
+            # without color support, so turtles all the way down: we return
+            # another instance.
+            return NullCallableString()
+        return u''.join(args)
+
+
 def get_proxy_string(term, attr):
     """
     Proxy and return callable string for proxied attributes.
@@ -200,100 +355,6 @@ def get_proxy_string(term, attr):
     return _proxy_table.get(term_kind, {}).get(attr, None)
 
 
-class FormattingString(six.text_type):
-    r"""
-    A Unicode string which doubles as a callable.
-
-    This is used for terminal attributes, so that it may be used both
-    directly, or as a callable.  When used directly, it simply emits
-    the given terminal sequence.  When used as a callable, it wraps the
-    given (string) argument with the 2nd argument used by the class
-    constructor::
-
-        >>> style = FormattingString(term.bright_blue, term.normal)
-        >>> print(repr(style))
-        u'\x1b[94m'
-        >>> style('Big Blue')
-        u'\x1b[94mBig Blue\x1b(B\x1b[m'
-    """
-
-    def __new__(cls, *args):
-        """
-        Class constructor accepting 2 positional arguments.
-
-        :arg sequence: terminal attribute sequence.
-        :arg normal: terminating sequence for this attribute (optional).
-        """
-        assert 1 <= len(args) <= 2, args
-        new = six.text_type.__new__(cls, args[0])
-        new._normal = args[1] if len(args) > 1 else u''
-        return new
-
-    def __call__(self, *args):
-        """Return ``text`` joined by ``sequence`` and ``normal``."""
-        # Jim Allman brings us this convenience of allowing existing
-        # unicode strings to be joined as a call parameter to a formatting
-        # string result, allowing nestation:
-        #
-        # >>> t.red('This is ', t.bold('extremely'), ' dangerous!')
-        for idx, ucs_part in enumerate(args):
-            if not isinstance(ucs_part, six.string_types):
-                raise TypeError("Positional argument #{idx} is {is_type} "
-                                "expected any of {expected_types}: "
-                                "{ucs_part!r}".format(
-                                    idx=idx, ucs_part=ucs_part,
-                                    is_type=type(ucs_part),
-                                    expected_types=six.string_types,
-                                ))
-        postfix = u''
-        if self and self._normal:
-            postfix = self._normal
-            _refresh = self._normal + self
-            args = [_refresh.join(ucs_part.split(self._normal))
-                    for ucs_part in args]
-
-        return self + u''.join(args) + postfix
-
-
-class NullCallableString(six.text_type):
-    """
-    A dummy callable Unicode alternative to :class:`FormattingString`.
-
-    This is used for colors on terminals that do not support colors,
-    it is just a basic form of unicode that may also act as a callable.
-    """
-
-    def __new__(cls):
-        """Class constructor."""
-        new = six.text_type.__new__(cls, u'')
-        return new
-
-    def __call__(self, *args):
-        """
-        Allow empty string to be callable, returning given string, if any.
-
-        When called with an int as the first arg, return an empty Unicode. An
-        int is a good hint that I am a :class:`ParameterizingString`, as there
-        are only about half a dozen string-returning capabilities listed in
-        terminfo(5) which accept non-int arguments, they are seldom used.
-
-        When called with a non-int as the first arg (no no args at all), return
-        the first arg, acting in place of :class:`FormattingString` without
-        any attributes.
-        """
-        if not args or isinstance(args[0], int):
-            # As a NullCallableString, even when provided with a parameter,
-            # such as t.color(5), we must also still be callable, fe:
-            #
-            # >>> t.color(5)('shmoo')
-            #
-            # is actually simplified result of NullCallable()() on terminals
-            # without color support, so turtles all the way down: we return
-            # another instance.
-            return NullCallableString()
-        return u''.join(args)
-
-
 def split_compound(compound):
     """
     Split compound formating string into segments.
@@ -304,6 +365,7 @@ def split_compound(compound):
     :arg str compound: a string that may contain compounds, separated by
         underline (``_``).
     :rtype: list
+    :returns: List of formating string segments
     """
     merged_segs = []
     # These occur only as prefixes, so they can always be merged:
@@ -327,11 +389,11 @@ def resolve_capability(term, attr):
        given :attr:`~.Terminal.kind`.
     :rtype: str
     """
-    # Decode sequences as latin1, as they are always 8-bit bytes, so when
-    # b'\xff' is returned, this must be decoded to u'\xff'.
     if not term.does_styling:
         return u''
-    val = curses.tigetstr(term._sugar.get(attr, attr))
+    val = curses.tigetstr(term._sugar.get(attr, attr))  # pylint: disable=protected-access
+    # Decode sequences as latin1, as they are always 8-bit bytes, so when
+    # b'\xff' is returned, this is decoded as u'\xff'.
     return u'' if val is None else val.decode('latin1')
 
 
@@ -351,25 +413,41 @@ def resolve_color(term, color):
         otherwise :class:`FormattingString`.
     :rtype: :class:`NullCallableString` or :class:`FormattingString`
     """
+    # pylint: disable=protected-access
     if term.number_of_colors == 0:
         return NullCallableString()
 
-    # NOTE(erikrose): Does curses automatically exchange red and blue and cyan
-    # and yellow when a terminal supports setf/setb rather than setaf/setab?
-    # I'll be blasted if I can find any documentation. The following
-    # assumes it does: to terminfo(5) describes color(1) as COLOR_RED when
-    # using setaf, but COLOR_BLUE when using setf.
-    color_cap = (term._background_color if 'on_' in color else
-                 term._foreground_color)
+    # fg/bg capabilities terminals that support 0-256+ colors.
+    vga_color_cap = (term._background_color if 'on_' in color else
+                     term._foreground_color)
 
-    # curses constants go up to only 7, so add an offset to get at the
-    # bright colors at 8-15:
-    offset = 8 if 'bright_' in color else 0
     base_color = color.rsplit('_', 1)[-1]
+    if base_color in CGA_COLORS:
+        # curses constants go up to only 7, so add an offset to get at the
+        # bright colors at 8-15:
+        offset = 8 if 'bright_' in color else 0
+        base_color = color.rsplit('_', 1)[-1]
+        attr = 'COLOR_%s' % (base_color.upper(),)
+        fmt_attr = vga_color_cap(getattr(curses, attr) + offset)
+        return FormattingString(fmt_attr, term.normal)
 
-    attr = 'COLOR_%s' % (base_color.upper(),)
-    fmt_attr = color_cap(getattr(curses, attr) + offset)
-    return FormattingString(fmt_attr, term.normal)
+    assert base_color in X11_COLORNAMES_TO_RGB, (
+        'color not known', base_color)
+    rgb = X11_COLORNAMES_TO_RGB[base_color]
+
+    # downconvert X11 colors to CGA, EGA, or VGA color spaces
+    if term.number_of_colors <= 256:
+        fmt_attr = vga_color_cap(term.rgb_downconvert(*rgb))
+        return FormattingString(fmt_attr, term.normal)
+
+    # Modern 24-bit color terminals are written pretty basically.  The
+    # foreground and background sequences are:
+    # - ^[38;2;<r>;<g>;<b>m
+    # - ^[48;2;<r>;<g>;<b>m
+    fgbg_seq = ('48' if 'on_' in color else '38')
+    assert term.number_of_colors == 1 << 24
+    fmt_attr = u'\x1b[' + fgbg_seq + ';2;{0};{1};{2}m'
+    return FormattingString(fmt_attr.format(*rgb), term.normal)
 
 
 def resolve_attribute(term, attr):
@@ -400,7 +478,7 @@ def resolve_attribute(term, attr):
     # call for each compounding section, joined and returned as
     # a completed completed FormattingString.
     formatters = split_compound(attr)
-    if all(fmt in COMPOUNDABLES for fmt in formatters):
+    if all(fmt in (COLORS | COMPOUNDABLES) for fmt in formatters):
         resolution = (resolve_attribute(term, fmt) for fmt in formatters)
         return FormattingString(u''.join(resolution), term.normal)
 
@@ -413,7 +491,8 @@ def resolve_attribute(term, attr):
         # and, for special terminals, such as 'screen', provide a Proxy
         # ParameterizingString for attributes they do not claim to support,
         # but actually do! (such as 'hpa' and 'vpa').
-        proxy = get_proxy_string(term, term._sugar.get(attr, attr))
+        proxy = get_proxy_string(term,
+                                 term._sugar.get(attr, attr))  # pylint: disable=protected-access
         if proxy is not None:
             return proxy
 
